import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Queue;

/**
 * Sonorizador TCP/IP
 * Tem um arranjo de executores, cada um de eles vai tocar um instrumento.
 * Cada executor esta associado a um instrumento.
 *
 */
public class Sonorizador extends Thread implements Notas, Instrumentos {
	private Executor executores[];

	//Escalas
	public final static int duracoes[] = { 125, 150, 175, 200, 225, 250, 500,
			750 };
	public final static int scale[] = { CS4, DS4, FS4, GS4, CS5, DS5, FS5, GS5 };
	public static int[] scalaTemperada = { C3, CF3, D3, DF3, E3, F3, FF3, G3,
			GF3, A3, AF3, B3, C4 };
	public static int[] scala4 = { C4, D4, E4, F4, G4, A4, B4, C5 };

	//Instrumentos
	public final static int instruments[] = { 0, 1, 2, CHARANG, 4, VIOLIN, 6,
			7, DRUM, 9, 10, RECORDER, 12, 13 };

	/**
	 * Construtor, inicializa cada executor
	 */
	public Sonorizador() {
		executores = new Executor[14];
		for (int i = 0; i < executores.length; i++) {
			executores[i] = new Executor();
			executores[i].start();
			executores[i].instrument = instruments[i];
		}
	}

	/**
	 * Cada 5 segundos executamos netstat para olhar o estado das conexoes.
	 */
	public void run() {
		
		int netstatSleepTime = 5000;

		while (true) {
			int pitch = 64, duration = 250, velocity = 80;

			ArrayList<Connection> lista = Util.netstat();

			//Iteramos todas as conexoes
			for (Connection connection : lista) {
	
				String[] nums = connection.destIp.split("\\."); //pegamos a IP de destino
				for (int i = 0; i < nums.length; i++) {
					int indexScale, indexExecutor;
					indexScale = Integer.parseInt(nums[i]) % scale.length; // S pegamos valores entre 0 e o comprimento da escala 
					indexExecutor = connection.state;

					duration = duracoes[connection.srcPort % duracoes.length];
					pitch = scale[indexScale];
					executores[indexExecutor].addNote(pitch, duration, velocity);
				}

			}
			try {
				sleep(netstatSleepTime);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}
		}
	}

	public static void main(String[] args) {
		new Sonorizador().start();
	}
}
